/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// THIS CODE IS AUTOMATICALLY GENERATED.  DO NOT EDIT.

package org.apache.kafka.common.metadata;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import org.apache.kafka.common.errors.UnsupportedVersionException;
import org.apache.kafka.common.protocol.ApiMessage;
import org.apache.kafka.common.protocol.MessageSizeAccumulator;
import org.apache.kafka.common.protocol.MessageUtil;
import org.apache.kafka.common.protocol.ObjectSerializationCache;
import org.apache.kafka.common.protocol.Readable;
import org.apache.kafka.common.protocol.Writable;
import org.apache.kafka.common.protocol.types.Field;
import org.apache.kafka.common.protocol.types.RawTaggedField;
import org.apache.kafka.common.protocol.types.RawTaggedFieldWriter;
import org.apache.kafka.common.protocol.types.Schema;
import org.apache.kafka.common.protocol.types.Type;
import org.apache.kafka.common.utils.ByteUtils;


public class FeatureLevelRecord implements ApiMessage {
    String name;
    short minFeatureLevel;
    short maxFeatureLevel;
    private List<RawTaggedField> _unknownTaggedFields;
    
    public static final Schema SCHEMA_0 =
        new Schema(
            new Field("name", Type.STRING, "The feature name."),
            new Field("min_feature_level", Type.INT16, "The current finalized minimum feature level of this feature for the cluster."),
            new Field("max_feature_level", Type.INT16, "The current finalized maximum feature level of this feature for the cluster.")
        );
    
    public static final Schema[] SCHEMAS = new Schema[] {
        SCHEMA_0
    };
    
    public static final short LOWEST_SUPPORTED_VERSION = 0;
    public static final short HIGHEST_SUPPORTED_VERSION = 0;
    
    public FeatureLevelRecord(Readable _readable, short _version) {
        read(_readable, _version);
    }
    
    public FeatureLevelRecord() {
        this.name = "";
        this.minFeatureLevel = (short) 0;
        this.maxFeatureLevel = (short) 0;
    }
    
    @Override
    public short apiKey() {
        return 12;
    }
    
    @Override
    public short lowestSupportedVersion() {
        return 0;
    }
    
    @Override
    public short highestSupportedVersion() {
        return 0;
    }
    
    @Override
    public void read(Readable _readable, short _version) {
        {
            int length;
            length = _readable.readShort();
            if (length < 0) {
                throw new RuntimeException("non-nullable field name was serialized as null");
            } else if (length > 0x7fff) {
                throw new RuntimeException("string field name had invalid length " + length);
            } else {
                this.name = _readable.readString(length);
            }
        }
        this.minFeatureLevel = _readable.readShort();
        this.maxFeatureLevel = _readable.readShort();
        this._unknownTaggedFields = null;
    }
    
    @Override
    public void write(Writable _writable, ObjectSerializationCache _cache, short _version) {
        int _numTaggedFields = 0;
        {
            byte[] _stringBytes = _cache.getSerializedValue(name);
            _writable.writeShort((short) _stringBytes.length);
            _writable.writeByteArray(_stringBytes);
        }
        _writable.writeShort(minFeatureLevel);
        _writable.writeShort(maxFeatureLevel);
        RawTaggedFieldWriter _rawWriter = RawTaggedFieldWriter.forFields(_unknownTaggedFields);
        _numTaggedFields += _rawWriter.numFields();
        if (_numTaggedFields > 0) {
            throw new UnsupportedVersionException("Tagged fields were set, but version " + _version + " of this message does not support them.");
        }
    }
    
    @Override
    public void addSize(MessageSizeAccumulator _size, ObjectSerializationCache _cache, short _version) {
        int _numTaggedFields = 0;
        {
            byte[] _stringBytes = name.getBytes(StandardCharsets.UTF_8);
            if (_stringBytes.length > 0x7fff) {
                throw new RuntimeException("'name' field is too long to be serialized");
            }
            _cache.cacheSerializedValue(name, _stringBytes);
            _size.addBytes(_stringBytes.length + 2);
        }
        _size.addBytes(2);
        _size.addBytes(2);
        if (_unknownTaggedFields != null) {
            _numTaggedFields += _unknownTaggedFields.size();
            for (RawTaggedField _field : _unknownTaggedFields) {
                _size.addBytes(ByteUtils.sizeOfUnsignedVarint(_field.tag()));
                _size.addBytes(ByteUtils.sizeOfUnsignedVarint(_field.size()));
                _size.addBytes(_field.size());
            }
        }
        if (_numTaggedFields > 0) {
            throw new UnsupportedVersionException("Tagged fields were set, but version " + _version + " of this message does not support them.");
        }
    }
    
    @Override
    public boolean equals(Object obj) {
        if (!(obj instanceof FeatureLevelRecord)) return false;
        FeatureLevelRecord other = (FeatureLevelRecord) obj;
        if (this.name == null) {
            if (other.name != null) return false;
        } else {
            if (!this.name.equals(other.name)) return false;
        }
        if (minFeatureLevel != other.minFeatureLevel) return false;
        if (maxFeatureLevel != other.maxFeatureLevel) return false;
        return MessageUtil.compareRawTaggedFields(_unknownTaggedFields, other._unknownTaggedFields);
    }
    
    @Override
    public int hashCode() {
        int hashCode = 0;
        hashCode = 31 * hashCode + (name == null ? 0 : name.hashCode());
        hashCode = 31 * hashCode + minFeatureLevel;
        hashCode = 31 * hashCode + maxFeatureLevel;
        return hashCode;
    }
    
    @Override
    public FeatureLevelRecord duplicate() {
        FeatureLevelRecord _duplicate = new FeatureLevelRecord();
        _duplicate.name = name;
        _duplicate.minFeatureLevel = minFeatureLevel;
        _duplicate.maxFeatureLevel = maxFeatureLevel;
        return _duplicate;
    }
    
    @Override
    public String toString() {
        return "FeatureLevelRecord("
            + "name=" + ((name == null) ? "null" : "'" + name.toString() + "'")
            + ", minFeatureLevel=" + minFeatureLevel
            + ", maxFeatureLevel=" + maxFeatureLevel
            + ")";
    }
    
    public String name() {
        return this.name;
    }
    
    public short minFeatureLevel() {
        return this.minFeatureLevel;
    }
    
    public short maxFeatureLevel() {
        return this.maxFeatureLevel;
    }
    
    @Override
    public List<RawTaggedField> unknownTaggedFields() {
        if (_unknownTaggedFields == null) {
            _unknownTaggedFields = new ArrayList<>(0);
        }
        return _unknownTaggedFields;
    }
    
    public FeatureLevelRecord setName(String v) {
        this.name = v;
        return this;
    }
    
    public FeatureLevelRecord setMinFeatureLevel(short v) {
        this.minFeatureLevel = v;
        return this;
    }
    
    public FeatureLevelRecord setMaxFeatureLevel(short v) {
        this.maxFeatureLevel = v;
        return this;
    }
}
